<?php
    require_once KCONFIG . '/classes/classKGatewayUtil.php';
    require_once KCONFIG . '/classes/KGatewayYaml.php';

    $AjaxFunctions['showCodecProfiles'] = 'showCodecProfiles';
    $AjaxFunctions['addCodecProfile'] = 'addCodecProfile';
    $AjaxFunctions['editCodecProfile'] = 'editCodecProfile';
    $AjaxFunctions['saveCodecProfile'] = 'saveCodecProfile';
    $AjaxFunctions['deleteCodecProfile'] = 'deleteCodecProfile';

    function showCodecProfiles( )
    {
        require_once KROOT . '/classes/KLicense.php';

        $app = KSystem::getInstance( );
        $kyaml = new KGatewayYaml( );
        $yaml = $kyaml->getYaml( );

        $buffer = "<fieldset><legend>{$app->translate( 'gateway-codec-profile' )}</legend><div id='gateway-codec-profile-container'><input type='button' id='gateway-codec-profile-button-new' value='{$app->translate( 'gateway-codec-profile-new' )}' /><br /><br />";

        if( array_key_exists( 'CodecProfiles', $yaml ) && count( $yaml['CodecProfiles'] ) > 0 )
        {
            $manager = new KLicenseManager( );
            $noG729license = !$manager->anyLicenseHasType( KLicense::TYPE_G729 );

            $buffer .= "<table id='gateway-codec-profile-table' class='gateway-table'><thead><tr><th>{$app->translate( 'gateway-codec-profile-name' )}</th><th>{$app->translate( 'gateway-codec-profile-list' )}</th><th class='gateway-table-options'>{$app->translate( 'options' )}</th></tr></thead>";
            foreach( $yaml['CodecProfiles'] as $idx => $profile )
            {
                $codecs = array( );
                foreach( $profile['CodecList'] as $codec )
                {
                    if( $codec == 'G729' && $noG729license )
                    {
                        $codecs[] = '<span class=\'unavailable\'>G729</span>';
                    }
                    else
                    {
                        $codecs[] = $codec;
                    }
                }
                $codecList = implode( ',', $codecs );
                $buffer .= "<tr><td>{$profile['Name']}</td><td>{$codecList}</td>";
                $buffer .= "<td class='gateway-table-options'><img class='gateway-codec-img gateway-codec-profile-edit' src='/imagens/editark.gif' title='{$app->translate( 'edit' )}' /><img class='gateway-codec-img gateway-codec-profile-delete' src='/imagens/lixeira.gif' title='{$app->translate( 'delete' )}' /></td></tr>";
            }
            $buffer .= '</table>';
        }
        else
        {
            $buffer .= "<p>{$app->translate( 'gateway-codec-profile-empty-list' )}</p>";
        }

        $buffer .= '</div></fieldset>';
        return $buffer;
    }

    function addCodecProfile( )
    {
        return _codecProfileForm( );
    }

    function editCodecProfile( )
    {
        $name = array_key_exists( 'name', $_POST ) ? $_POST['name'] : null;
        $app = KSystem::getInstance( );

        $kyaml = new KGatewayYaml( );
        $yaml = $kyaml->getYaml( );

        //TODO: create a function in KGatewayYaml to search for a codec
        // profile
        if( !is_null( $name ) && array_key_exists( 'CodecProfiles', $yaml ) && count( $yaml['CodecProfiles'] ) > 0 )
        {
            foreach( $yaml['CodecProfiles'] as &$profile )
            {
                if( $profile['Name'] == $name )
                {
                    return _codecProfileForm( $profile );
                }
            }
        }

        return $app->translate( 'gateway-codec-profile-error-not-found' );
    }

    function saveCodecProfile( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-codec-profile-save-error' ) );

        try
        {
            if( array_key_exists( 'name', $_POST ) && array_key_exists( 'oldName', $_POST ) && array_key_exists( 'codecList', $_POST ) )
            {
                $kyaml = new KGatewayYaml( );
                $yaml = $kyaml->getYaml( );

                if( !array_key_exists( 'CodecProfiles', $yaml ) )
                {
                    $yaml['CodecProfiles'] = array( );
                }

                //TODO: create KGatewayYaml function to set CodecProfile
                $renamed = false;
                $duplicated = false;
                $new = strlen( $_POST['oldName'] ) == 0;

                if( !$new && $_POST['oldName'] != $_POST['name'] )
                {
                    $renamed = true;
                }
                if( $new || $renamed )
                {
                    // if it is new or renamed, check if $_POST['name'] already
                    // exists -- duplicate key check
                    foreach( $yaml['CodecProfiles'] as &$profile )
                    {
                        if( $profile['Name'] == $_POST['name'] )
                        {
                            $duplicated = true;
                            break;
                        }
                    }
                }
                if( $duplicated )
                {
                    $reply['message'] = $app->translate( 'gateway-codec-profile-save-error-duplicated' );
                }
                else
                {
                    if( $new )
                    {
                        // append new
                        $yaml['CodecProfiles'][] = array( 'Name' => $_POST['name'], 'CodecList' => $_POST['codecList'] );
                    }
                    else if( $renamed )
                    {
                        // update
                        foreach( $yaml['CodecProfiles'] as $idx => &$profile )
                        {
                            if( $profile['Name'] == $_POST['oldName'] )
                            {
                                $profile['Name'] = $_POST['name'];
                                $profile['CodecList'] = $_POST['codecList'];
                                break;
                            }
                        }
                        // update any route that referrer to the old name
                        foreach( $yaml['Routes'] as &$route )
                        {
                            if( $route['CodecProfile'] == $_POST['oldName'] )
                            {
                                $route['CodecProfile'] = $_POST['name'];
                            }
                        }
                    }
                    else
                    {
                        // update *only*
                        foreach( $yaml['CodecProfiles'] as &$profile )
                        {
                            if( $profile['Name'] == $_POST['name'] )
                            {
                                $profile['CodecList'] = $_POST['codecList'];
                                break;
                            }
                        }
                    }
                    $kyaml->setYaml( $yaml );
                    $kyaml->save( );
                    $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-codec-profile-saved' ) );
                }
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    function deleteCodecProfile( )
    {
        require_once KROOT . '/json_util.php';
        $app = KSystem::getInstance( );

        $reply = array( 'success' => false, 'message' => $app->translate( 'gateway-codec-profile-delete-error' ) );

        try
        {
            $kyaml = new KGatewayYaml( );
            //TODO: create a function to remove CodecProfile inside KGatewayYaml
            $yaml = $kyaml->getYaml( );
            if( array_key_exists( 'CodecProfiles', $yaml ) && count( $yaml['CodecProfiles'] ) > 0 )
            {
                // search the right profile and remove it from CodecProfiles
                foreach( $yaml['CodecProfiles'] as $idx => $profile )
                {
                    if( $profile['Name'] == $_POST['name'] )
                    {
                        $inUse = false;
                        foreach( $yaml['Routes'] as $route )
                        {
                            if( $route['CodecProfile'] == $profile['Name'] )
                            {
                                $inUse = true;
                                break;
                            }
                        }
                        if( $inUse )
                        {
                            $reply['message'] = $app->translate( 'gateway-codec-profile-delete-error-inuse' );
                            break;
                        }
                        unset( $yaml['CodecProfiles'][$idx] );
                        $kyaml->setYaml( $yaml );
                        $kyaml->save( );
                        $reply = array( 'success' => true, 'message' => $app->translate( 'gateway-codec-profile-deleted' ) );
                        break;
                    }
                }
            }
        }
        catch( Exception $e )
        {
            $reply['message'] = $app->translate( $e->getMessage( ) );
        }

        return JSONencode( $reply );
    }

    function _codecProfileForm( $data = array() )
    {
        require_once KROOT . '/classes/KLicense.php';

        $app = KSystem::getInstance( );

        $name = array_key_exists( 'Name', $data ) ? $data['Name'] : '';
        $codecList = array_key_exists( 'CodecList', $data ) ? $data['CodecList'] : array( );
        //TODO: Codecs list *should* be available from some global source
        // (KGatewayUtil,KGatewayYaml,KSystemYaml (sip configuration for k3l)?)
        $codecs = array( 'G711A', 'G711U', 'G729', 'GSM', 'DVI4', 'iLBC' );

        $buffer = '<form name=\'gateway-codec-profile-form\' id=\'gateway-codec-profile-form\'>';
        $buffer .= '<table id=\'gateway-codec-profile-form-table\'>';
        $buffer .= "<tr><td>{$app->translate( 'gateway-codec-profile-name' )}:</td><td><input type='hidden' id='gateway-codec-profile-old-name' value='{$name}' /><input type='text' id='gateway-codec-profile-name' value='{$name}' /></td></tr>";
        $buffer .= "<tr><td>{$app->translate( 'gateway-codec-profile-list' )}: <img src='/imagens/tooltip.png' class='gateway-tooltip' alt='tooltip' khelp='{$app->translate( 'gateway-codec-profile-help-list' )}' /><br /><a id='gateway-codec-profile-select-all' href='#'>{$app->translate( 'all' )}</a> - <a id='gateway-codec-profile-select-none' href='#'>{$app->translate( 'none' )}</a></td><td><ul class='ui-sortable' id='gateway-codec-profile-list-container'>";
        // reorder $codecs using the $codecList as first elements
        if( count( $codecList ) > 0 )
        {
            $codecsDisabled = array_diff( $codecs, $codecList );
            $codecs = array_merge( $codecList, $codecsDisabled );
            unset( $codecsDisabled );
        }

        $manager = new KLicenseManager( );
        $noG729license = !$manager->anyLicenseHasType( KLicense::TYPE_G729 );

        foreach( $codecs as $codec )
        {
            if( $codec == 'G729' && $noG729license )
            {
                $cls = ' unavailable';
                $extra = "&nbsp;<img class='gateway-tooltip' khelp='{$app->translate( 'gateway-codec-profile-help-license-required' )}' alt='tooltip' src='/imagens/tooltip.png' />";
            }
            else
            {
                $cls = '';
                $extra = '';
            }
            $checked = in_array( $codec, $codecList ) ? ' checked' : '';
            $buffer .= "<li class='ui-state-default {$cls}'><input type='checkbox' class='gateway-codec-profile-list' value='{$codec}'{$checked}>{$codec}{$extra}</li>";
        }
        $buffer .= "</ul></td></tr>";
        $buffer .= "</table><p><input type='button' id='gateway-codec-profile-save' value='{$app->translate( 'save' )}' /><input type='button' id='gateway-codec-profile-back' value='{$app->translate( 'back' )}' /></p></form>";

        return $buffer;
    }
